function detC2 = comp_C(Aq,Kmax,flag_comp)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes det(\Gamma) for a given AR model                 
% The \Gamma matrix from the paper is called here C                       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%INPUT:
%Aq         : "true" AR model
%Kmax       : max. order for the fitted AR model
%flag_comp  : 1 - plots figures for comparing the two methods used to 
%           :   - computes det(C) with two different methods
%           : 0 - no plots
%           :   - computes det(C) by using one single method
%OUTPUT:
%detC2      : vector with the computed values of det(C)  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all

% q = model order
q = length(Aq)-1;

%the roots of the model
if flag_comp
    figure
    zplane(roots(Aq));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               First method for computing det(C)                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%S matrix
S = zeros(q+1, q+1);
for i = 2:(q+1),
	for j = 2:i,
		S(i,j) = Aq(1, i-j+1);
	end
end

%T matrix
T = zeros(q+1, q+1);
for i = 1:(q+1),
	for j = 1:(q+2-i),
		T(i,j) = Aq(1, i+j-1);
	end
end

%Find the autocovariances at lags 0,...,q
ss = zeros(q+1, 1);
ss(1) = 1.0; %driven noise has unitary variance
arr = (S+T)\ss;


for j=q+1:Kmax,
    vecc = arr(end:-1:end-q+1);
    cj = -Aq(2:end)*vecc;
    arr = [arr; cj];
end

    figure
    plot([0:length(arr)-1],arr);
    xlabel('Lag k');
    ylabel('r(k)');

%Compute det(C) when the order of the fitted AR model takes values 
%from 1 to Kmax
detC = zeros(Kmax,1);
detC(1) = arr(1);
for k=1:Kmax-1,
    C = toeplitz(arr(1:k+1));
    detC(k+1) = det(C);
end
end %if flag_comp

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Second method for computing det(C)                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%First compute the reflection coefficients starting from the coeff. of the 
%"true" AR model
%For the algorithm applied, see S. Haykin, Adaptive filter theory, 2002,
%Inverse Levinson-Durbin algorithm, p. 155-156 

refcoeff = zeros(q,1);
al = Aq(2:end)';
refcoeff(q) = al(end);

for j=q-1:-1:1,
    kk = refcoeff(j+1);
    al = (al(1:end-1) - kk*al(end-1:-1:1))/(1-kk^2);
    refcoeff(j) = al(end);
end

%Compute autocorr. at lag 0 starting from the reflection coefficients
%We use the following formula from S. Haykin, Adaptive filter theory, 2002,
%p. 154:
%$P_m = P_{m-1}(1-k_m^2)$, where $k_m$ denotes the reflection coeff. 
%and we have $P_q=1$ because the var. of the driven noise is one.
r0 = 1/prod(ones(q,1) - refcoeff.^2);

%Compute det(C) with formula from S. Kay, Recursive maximum likelihood 
%estimation of autoregressive processes, IEEE TrSP, vol. 31, no. 1,
%Feb. 1983, p. 56-65 (formula is at p. 57)
%See also S. Haykin, Adaptive filter theory, 2002, p. 171-172
%For the calculation of P, we use the same formula as the one applied 
%in the calculation of r0
P = r0;
detC2 = zeros(Kmax,1);
detC2(1) = r0;
for k=1:Kmax-1,
    if k<q+1,
        P = P*(1-refcoeff(k)^2);
    end
    detC2(k+1) = detC2(k)*P;
end

if flag_comp,
    figure
    plot((1:Kmax),detC2);
    xlabel('Order p');
	('det(C)');

    figure
    plot((1:Kmax),detC-detC2);
    xlabel('Order p');
    ylabel('Difference');
    legend('Difference between results obtained with the two methods');
end


